@@ -87,6 +87,8 @@ void OOMErrorHandler(const char* location, const v8::OOMDetails& details);
   V(ERR_FS_CP_SOCKET, Error)                                                   \
   V(ERR_FS_CP_FIFO_PIPE, Error)                                                \
   V(ERR_FS_CP_UNKNOWN, Error)                                                  \
+  V(ERR_HEAP_PROFILE_HAVE_BEEN_STARTED, Error)                                 \
+  V(ERR_HEAP_PROFILE_NOT_STARTED, Error)                                       \
   V(ERR_ILLEGAL_CONSTRUCTOR, Error)                                            \
   V(ERR_INVALID_ADDRESS, Error)                                                \
   V(ERR_INVALID_ARG_VALUE, TypeError)                                          \
@@ -104,12 +106,13 @@ void OOMErrorHandler(const char* location, const v8::OOMDetails& details);
   V(ERR_INVALID_URL_PATTERN, TypeError)                                        \
   V(ERR_INVALID_URL_SCHEME, TypeError)                                         \
   V(ERR_LOAD_SQLITE_EXTENSION, Error)                                          \
-  V(ERR_MEMORY_ALLOCATION_FAILED, Error)                                       \
+  V(ERR_MEMORY_ALLOCATION_FAILED, RangeError)                                  \
   V(ERR_MESSAGE_TARGET_CONTEXT_UNAVAILABLE, Error)                             \
   V(ERR_MISSING_ARGS, TypeError)                                               \
   V(ERR_MISSING_PASSPHRASE, TypeError)                                         \
   V(ERR_MISSING_PLATFORM_FOR_WORKER, Error)                                    \
   V(ERR_MODULE_NOT_FOUND, Error)                                               \
+  V(ERR_MODULE_NOT_INSTANTIATED, Error)                                        \
   V(ERR_MODULE_LINK_MISMATCH, TypeError)                                       \
   V(ERR_NON_CONTEXT_AWARE_DISABLED, Error)                                     \
   V(ERR_OPERATION_FAILED, TypeError)                                           \
@@ -137,7 +140,7 @@ void OOMErrorHandler(const char* location, const v8::OOMDetails& details);
 #define V(code, type)                                                          \
   template <typename... Args>                                                  \
   inline v8::Local<v8::Object> code(                                           \
-      v8::Isolate* isolate, const char* format, Args&&... args) {              \
+      v8::Isolate* isolate, std::string_view format, Args&&... args) {         \
     std::string message;                                                       \
     if (sizeof...(Args) == 0) {                                                \
       message = format;                                                        \
@@ -162,17 +165,18 @@ void OOMErrorHandler(const char* location, const v8::OOMDetails& details);
   }                                                                            \
   template <typename... Args>                                                  \
   inline void THROW_##code(                                                    \
-      v8::Isolate* isolate, const char* format, Args&&... args) {              \
+      v8::Isolate* isolate, std::string_view format, Args&&... args) {         \
     isolate->ThrowException(                                                   \
         code(isolate, format, std::forward<Args>(args)...));                   \
   }                                                                            \
   template <typename... Args>                                                  \
   inline void THROW_##code(                                                    \
-      Environment* env, const char* format, Args&&... args) {                  \
+      Environment* env, std::string_view format, Args&&... args) {             \
     THROW_##code(env->isolate(), format, std::forward<Args>(args)...);         \
   }                                                                            \
   template <typename... Args>                                                  \
-  inline void THROW_##code(Realm* realm, const char* format, Args&&... args) { \
+  inline void THROW_##code(                                                    \
+      Realm* realm, std::string_view format, Args&&... args) {                 \
     THROW_##code(realm->isolate(), format, std::forward<Args>(args)...);       \
   }
 ERRORS_WITH_CODE(V)
@@ -229,6 +233,7 @@ ERRORS_WITH_CODE(V)
   V(ERR_MISSING_PLATFORM_FOR_WORKER,                                           \
     "The V8 platform used by this instance of Node does not support "          \
     "creating Workers")                                                        \
+  V(ERR_MODULE_NOT_INSTANTIATED, "Module is not instantiated")                 \
   V(ERR_NON_CONTEXT_AWARE_DISABLED,                                            \
     "Loading non context-aware native addons has been disabled")               \
   V(ERR_SCRIPT_EXECUTION_INTERRUPTED,                                          \
@@ -254,10 +259,8 @@ PREDEFINED_ERROR_MESSAGES(V)
 // Errors with predefined non-static messages
 inline void THROW_ERR_SCRIPT_EXECUTION_TIMEOUT(Environment* env,
                                                int64_t timeout) {
-  std::ostringstream message;
-  message << "Script execution timed out after ";
-  message << timeout << "ms";
-  THROW_ERR_SCRIPT_EXECUTION_TIMEOUT(env, message.str().c_str());
+  THROW_ERR_SCRIPT_EXECUTION_TIMEOUT(
+      env, "Script execution timed out after %dms", timeout);
 }
 
 inline void THROW_ERR_REQUIRE_ASYNC_MODULE(
@@ -272,14 +275,14 @@ inline void THROW_ERR_REQUIRE_ASYNC_MODULE(
   if (!parent_filename.IsEmpty() && parent_filename->IsString()) {
     Utf8Value utf8(env->isolate(), parent_filename);
     message += "\n  From ";
-    message += utf8.out();
+    message += utf8.ToStringView();
   }
   if (!filename.IsEmpty() && filename->IsString()) {
     Utf8Value utf8(env->isolate(), filename);
     message += "\n  Requiring ";
-    message += +utf8.out();
+    message += utf8.ToStringView();
   }
-  THROW_ERR_REQUIRE_ASYNC_MODULE(env, message.c_str());
+  THROW_ERR_REQUIRE_ASYNC_MODULE(env, message);
 }
 
 inline v8::Local<v8::Object> ERR_BUFFER_TOO_LARGE(v8::Isolate* isolate) {
@@ -292,12 +295,11 @@ inline v8::Local<v8::Object> ERR_BUFFER_TOO_LARGE(v8::Isolate* isolate) {
 }
 
 inline void THROW_ERR_SOURCE_PHASE_NOT_DEFINED(v8::Isolate* isolate,
-                                               v8::Local<v8::String> url) {
-  std::string message = std::string(*v8::String::Utf8Value(isolate, url));
+                                               const std::string& url) {
   return THROW_ERR_SOURCE_PHASE_NOT_DEFINED(
       isolate,
-      "Source phase import object is not defined for module %s",
-      message.c_str());
+      "Source phase import object is not defined for module '%s'",
+      url);
 }
 
 inline v8::Local<v8::Object> ERR_STRING_TOO_LONG(v8::Isolate* isolate) {
@@ -326,7 +328,7 @@ namespace errors {
 
 class TryCatchScope : public v8::TryCatch {
  public:
-  enum class CatchMode { kNormal, kFatal };
+  enum class CatchMode { kNormal, kFatal, kFatalRethrowStackOverflow };
 
   explicit TryCatchScope(Environment* env, CatchMode mode = CatchMode::kNormal)
       : v8::TryCatch(env->isolate()), env_(env), mode_(mode) {}
